/*
**  Copyright (c) 2003,2004,2005 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include "conf.h"
#include "parser.h"

struct _usb_conf usb_conf;

struct _usb_accepted_conf usb_accepted_conf[] = {
/*      name                type  default value                         value   */
	{"mntpoint",        CHAR, (char*)"/tmp/pam_usbXXXXXX",		&usb_conf.mntpoint},
	{"proc_basename",   CHAR, (char*)"/proc/scsi/usb-storage-%d/",	&usb_conf.proc_basename},
	{"local_keypath",   CHAR, (char*)"/.auth/",			&usb_conf.local_keypath},
	{"device_keypath",  CHAR, (char*)"/.auth/",			&usb_conf.device_keypath},
	{"pubkey",          CHAR, (char*)"id_pub",			&usb_conf.pubkey},
	{"snfile",          CHAR, (char*)"/etc/pam_usb/serials.conf", 	&usb_conf.snfile},
	{"fs",              CHAR, (char*)"ext2,vfat",			&usb_conf.fs},
	{"utmp",	        CHAR, (char*)"/var/run/utmp",		&usb_conf.utmp},
	{"mount_opts",	    CHAR, (char*)"",				&usb_conf.mount_opts},
	{"log_file",	    CHAR, (char*)"",				&usb_conf.log_file},
	{"force_device",    CHAR, (char*)"",				&usb_conf.force_device},
	{"local_consoles",  CHAR, (char*)"/dev/vc/,/dev/tty,:,tty",	&usb_conf.local_consoles},
	{"local_hosts",     CHAR, (char*)":",				&usb_conf.local_hosts},
	{"hostname",        CHAR, (char*)"",				&usb_conf.hostname},
	{"direct_open",	    INT,  (char*)"-1",				&usb_conf.direct_open},
	{"try_first_pass",  INT,  (char*)"-1",				&usb_conf.try_first_pass},
	{"use_first_pass",  INT,  (char*)"-1",				&usb_conf.use_first_pass},
	{"allow_remote",    INT,  (char*)"-1",				&usb_conf.allow_remote},
	{"check_device",    INT,  (char*)"-1",				&usb_conf.check_device},
	{"check_if_mounted",INT,  (char*)"-1",				&usb_conf.check_if_mounted},
	{"delete_mntpoint", INT,  (char*)"-1",				&usb_conf.delete_mntpoint},
	{"keep_mounted",    INT,  (char*)"-1",				&usb_conf.keep_mounted},
	{"sign_times",      INT,  (char*)"3",				&usb_conf.sign_times},
	{"debug",           INT,  (char*)"-1",				&usb_conf.debug},
	{"quiet",           INT,  (char*)"-1",				&usb_conf.quiet},
	{NULL,              0,      NULL,				NULL}
};

int put_arg_value (char *name, char *value)
{
	struct _usb_accepted_conf *conf_member = usb_accepted_conf;

	while (conf_member->name != NULL) {
		if (!strcmp (conf_member->name, name))
		{
			/* we found a winner */

			if (conf_member->type == CHAR) {
				*((char **)conf_member->value) = strdup(value);
			} else if (conf_member->type == INT) {
				*((int *)conf_member->value) = atoi(value);
			}
			return 1;
		}
		conf_member++;
	}
	return 0;
}

void put_default_values (void)
{
	struct _usb_accepted_conf *conf_member = usb_accepted_conf;

	while (conf_member->name) {
		switch (conf_member->type) {
		case CHAR:
			if (!*((char **)conf_member->value))
				put_arg_value (conf_member->name, 
					       conf_member->default_val);
			break;
		case INT:
			if (*((int *)conf_member->value) == 0) {
				put_arg_value (conf_member->name,
					       conf_member->default_val);
			}
			break;
		}

		conf_member++;
	}

}

/* FIXME: this function should update ac and av by removing parsed options.
   This would make possible to outside applications (usbadm) to parse arguments
   without adding extra code.
*/

void parse_args (int ac, const char **av)
{
	int i;
	char *name = NULL;
	char *value = NULL;

	for (i=0; i<ac; ++i)
	{
		/* Is there's no equal, then assume it's boolean */
		if (!strchr (av[i], '=')) {
			switch (av[i][0]) {
			case '!':
				put_arg_value ((char*)&(av[i][1]), "-1");
				break;
			default:
				put_arg_value ((char*)av[i], "1");
			}
			continue;
		}

		if ((name = get_from_to (av[i], 0, '=')) &&
		    (value = get_from_to (av[i], '=', '\0')))
			put_arg_value (name, value);

		free (name);
		free (value);
	}
}
