/*
**  Copyright (c) 2003,2004,2005 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <stdarg.h>
#include "pam.h"
#include "log.h"
#include "conf.h"

extern struct _usb_conf usb_conf;
FILE *debug_stream = NULL;

void log_init (void)
{
	/* no log file, print to stderr */
	if (!*(usb_conf.log_file)) {
		debug_stream = stderr;
		return;
	}

	/* log_file has been specified, open it and turn debug on */
	if (!(debug_stream = fopen (usb_conf.log_file, "a")))
		debug_stream = stderr;
	
	usb_conf.debug = 1;
}

void log_fini (void)
{
	if ((usb_conf.debug == 1) && (debug_stream != stderr))
		fclose (debug_stream);
}

void log_print(int level, char *format, ...)
{
	va_list ap;
	int log_level = 0;

	if (usb_conf.debug == 1)
		log_level = PLOG_DEBUG;
	if (!log_level && usb_conf.quiet == -1)
		log_level = PLOG_INFO;

	if (log_level < level)
		return;

	if (log_level == PLOG_INFO)
		fprintf(debug_stream, "* ");
	va_start(ap, format);
	vfprintf(debug_stream, format, ap);
	va_end(ap);
}
