/*
**  Copyright (c) 2003,2004,2005 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "usbadm.h"
#include <stdio.h>
#include <ncurses.h>
#include <term.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <security/pam_modules.h>
#include <security/_pam_macros.h>

#include <device.h>
#include <parser.h>
#include <dsa.h>

const EVP_CIPHER *get_cipher (char *algo)
{
	const EVP_CIPHER *enc = NULL;

	OpenSSL_add_all_algorithms();
	OpenSSL_add_all_ciphers();

	if (!algo || !strcmp(algo, "none"))
		enc = NULL;
	else
		if (!(enc = EVP_get_cipherbyname (algo)))
			printf ("[-] Invalid algorithm \"%s\".\n", algo);

	return enc;
}

int cipher (int ac, char **av)
{
	char *algo = NULL, *user, *file;
	FILE *f;
	DSA *key;

	if (ac < 4) {
		print_usage (av[1], av);
		return 1;
	}

	user = av[3];

	if (ac >= 5)
		algo = av[4];

	OpenSSL_add_all_ciphers();

	file = get_private_filename (user, av[2]);

	if (!(f = fopen (file, "r"))) {
		printf ("[-] Cannot open file %s\n", file);
		free (file);
		return 1;
	}


	printf ("[!] Importing the private key...\n");

	if (!(key = PEM_read_DSAPrivateKey (f, NULL, NULL, NULL))) {
		printf ("[-] Cannot import private key\n");
		free (file);
		fclose (f);
		return 1;
	}

	printf ("[+] Private key imported\n");

	if (!algo) {
		printf ("[!] Encrypting the private key may prevent someone to authenticate with\n");
		printf ("    your key. The drawback is that pam_usb will prompt you for password\n");
		printf ("    every time you authenticate.\n");

		algo = readline ("[?] Which algorithm want you to use ? (none/des3/twofish): ");
	}

	fclose (f);

	if (!(f = fopen (file, "w"))) {
		printf ("[-] Cannot reopen file %s for writing\n", file);
		free (file);
		return 1;
	}

	free (file);

	if (!PEM_write_DSAPrivateKey (f, key, get_cipher (algo), NULL, 0, NULL, NULL)) {
		printf ("[-] Cannot write private key using cipher %s\n", algo);
		fclose (f);
		return 1;
	}

	fclose (f);
	printf ("[+] Private key successfully written using cipher %s\n", algo);
	return 0;
}
