/*
**  Copyright (c) 2003,2004,2005 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include "usbhotplug.h"
#include "event.h"

#include <pwd.h>

extern char **environ;

int set_perm (void)
{
	struct passwd *pw = NULL;
	char *user = config.user;
	int found = 0;

	uid_t uid = 0;
	gid_t gid = 0;

	if (!user)
		return 1;

	setpwent();

	while ((pw = getpwent())) {
		if (!strcmp (user, pw->pw_name)) {
			uid = pw->pw_uid;
			gid = pw->pw_gid;

			++found;
			break;
		}
	}

	if (!found) {
		DEBUG ("Cannot find the entry for user %s, aborting\n", user);
		return 0;
	}

	if (setgid (gid) == -1)
		return 0;

	if (setegid (gid) == -1)
		return 0;

	if (setuid (uid) == -1)
		return 0;

	if (seteuid (uid) == -1)
		return 0;

	DEBUG ("Running as user %s [ uid:%u | gid:%u ]\n", user, uid, gid);

	return 1;
}

int retrieve_info (struct device_info *info)
{
	if (!(info->action = getenv ("ACTION")))
		return 0;

	if (!(info->product = getenv ("PRODUCT")))
		return 0;

	DEBUG ("Received hotplug event [%s] for product [%s]\n",
		info->action, info->product);

	if ((strcmp (info->action, "add") != 0) &&
		(strcmp (info->action, "remove") != 0)) {
		DEBUG ("Unknown event: %s\n", info->action);
		return 0;
	}

	return 1;
}

int handle_event (int status)
{
	if (!set_perm()) {
		DEBUG ("Cannot set permissions, aborting\n");
		return 0;
	}

	DEBUG ("Starting the %s handler\n", status ? "unlock" : "lock");

	if (status)
		return exec_handler (config.stop);
	else
		return exec_handler (config.start);
}

int exec_handler (char *handler)
{
	char **args = NULL;

	/* we got something like program arg1 arg2 */
	args = split (handler, ' ');

	DEBUG ("About to execute %s...\n", handler);

	if (execve (args[0], args, environ < 0)) {
		DEBUG ("Cannot execute the handler %s: %s\n", handler,
			strerror(errno));
		return 0;
	}

	return 1;
}
